import unittest
import numpy as np
from sklearn.metrics import pairwise_distances

from gene_trajectory.diffusion_map import diffusion_map, get_symmetrized_affinity_matrix


class DiffusionMapTestCase(unittest.TestCase):
    def test_diffusion_map(self):
        """
        Small test

        ```
        ce  = t(matrix(c(0,0, 2,3, 3,2,  5,0, 0,5), nrow = 2,
                    dimnames = list(c("D1","D2"), c("C1", "C2", "C3", "C4", "C5"))))
        cgd = as.matrix(dist(ce, method = "manhattan"))

        > diffusion.map(cgd, K=3, nEV = 3)
        ```
        returns
        ```
        $diffu.emb
                EV.1          EV.2        EV.3
        C1 -0.292406  6.666994e-17 -0.15219742
        C2 -0.292406 -5.888965e-02  0.11845799
        C3 -0.292406  5.888965e-02  0.11845799
        C4 -0.292406  2.515596e-01 -0.07091361
        C5 -0.292406 -2.515596e-01 -0.07091361

        $eigen.vals
        [1] 1.0000000 0.5219069 0.3861848

        attr(,"class")
        [1] "diffusion.map" "list"
        ```
        """
        ce = np.array([[0, 0], [2, 3], [3, 2], [5, 0], [0, 5]])
        gd = pairwise_distances(ce, metric='manhattan')

        diffu_emb, eigen_vals = diffusion_map(gd, k=3, n_ev=3, t=1)

        self.assertEqual(3, len(eigen_vals))

        np.testing.assert_array_almost_equal([1.0000000, 0.5219069, 0.3861848], eigen_vals, 6)
        # The eigenvectors have opposite direction between Python and R
        np.testing.assert_array_almost_equal(
            np.abs([-0.292406, -0.292406, -0.292406, -0.292406, -0.292406]),
            np.abs(diffu_emb[:, 0]), 6)
        np.testing.assert_array_almost_equal(
            np.abs([6.666994e-17, -5.888965e-02, 5.888965e-02, 2.515596e-01, -2.515596e-01]),
            np.abs(diffu_emb[:, 1]), 6)
        np.testing.assert_array_almost_equal(
            np.abs([-0.15219742, 0.11845799, 0.11845799, -0.07091361, -0.07091361]),
            np.abs(diffu_emb[:, 2]), 6)

    @staticmethod
    def test_get_symmetrized_affinity_matrix():
        ce = np.array([[0, 0], [2, 3], [3, 2]])
        gd = pairwise_distances(ce, metric='manhattan')

        affinity = get_symmetrized_affinity_matrix(gd, k=3)
        np.testing.assert_almost_equal(
            np.abs([
                [1., 0.367879, 0.367879],
                [0.367879, 1., 0.852144],
                [0.367879, 0.852144, 1.]
            ]), affinity, 6)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory
import numpy as np
import scipy.io as sio
from scipy.sparse import coo_matrix
from gene_trajectory.compute_gene_distance_cmd import cal_ot_mat


class DiffusionMapTestCase(unittest.TestCase):
    gdm = np.array([
        [0, 1, 2],
        [1, 0, 2],
        [2, 2, 1]])

    # Generated using R. Equal to get_gene_embedding() up to column signs
    gem = np.array([
        [.3, .1, .6],
        [.2, .3, .5],
        [.6, .2, .2]])

    expected_emd = np.array([
        [0.0, 0.8, 1.0],
        [0.8, 0.0, 0.9],
        [1.0, 0.9, 0.0]])

    gene_pairs = np.array([[0, 1], [1, 1], [2, 1]], dtype=int)

    def test_compute_gene_distance_cmd(self):
        with TemporaryDirectory() as d:
            np.savetxt(os.path.join(d, 'ot_cost.csv'), self.gdm, delimiter=',')
            sio.mmwrite(os.path.join(d, 'gene_expression.mtx'), coo_matrix(self.gem.T))

            cal_ot_mat(d, show_progress_bar=False)

            res = np.loadtxt(d + "/emd.csv", delimiter=",")
            np.testing.assert_almost_equal(res, self.expected_emd, decimal=6)

    def test_cal_ot_mat_gene_pairs(self):
        exp = self.expected_emd.copy()
        exp[0, 2] = exp[2, 0] = 900

        with TemporaryDirectory() as d:
            np.savetxt(os.path.join(d, 'ot_cost.csv'), self.gdm, delimiter=',')
            sio.mmwrite(os.path.join(d, 'gene_expression.mtx'), coo_matrix(self.gem.T))
            np.savetxt(os.path.join(d, 'gene_pairs.csv'), self.gene_pairs, fmt='%d', delimiter=',')

            cal_ot_mat(d, show_progress_bar=False)

            res = np.loadtxt(d + "/emd.csv", delimiter=",")
            np.testing.assert_almost_equal(res, exp, decimal=6)


if __name__ == '__main__':
    unittest.main()



import unittest

import numpy as np
import pandas as pd

from gene_trajectory.extract_gene_trajectory import get_gene_embedding, get_randow_walk_matrix, get_gene_pseudoorder, \
    extract_gene_trajectory
from tests.example_data import gene_names, gene_trajectories


class CoarseGrainTestCase(unittest.TestCase):
    gdm = np.array([
        [0.000000, 4.269544, 4.414329, 7.247308, 8.027305],
        [4.269544, 0.000000, 6.927429, 6.761171, 8.801408],
        [4.414329, 6.927429, 0.000000, 6.531824, 6.798761],
        [7.247308, 6.761171, 6.531824, 0.000000, 5.766003],
        [8.027305, 8.801408, 6.798761, 5.766003, 0.000000]])

    # Generated using R. Equal to get_gene_embedding() up to column signs
    gem = np.array([
        [0.17043479, 0.04309062, 0.02332959],
        [0.11273428, -0.10420329, 0.04087336],
        [0.02461065, 0.09818398, -0.06456570],
        [-0.12521597, -0.08515901, -0.07519815],
        [-0.20356317, 0.04675745, 0.08842471]])

    def test_get_gene_embedding(self):
        diffu_emb, eigen_vals = get_gene_embedding(self.gdm, k=3, n_ev=3)
        self.assertEqual(3, len(eigen_vals))

        np.testing.assert_array_almost_equal([0.4765176, 0.2777494, 0.2150586], eigen_vals, 6)

        # The eigenvectors have opposite direction between Python and R
        np.testing.assert_array_almost_equal(
            np.abs([0.170435, 0.112734, 0.024611, -0.125216, -0.203563]),
            np.abs(diffu_emb[:, 0]), 6)
        np.testing.assert_array_almost_equal(
            np.abs([0.043091, -0.104203, 0.098184, -0.085159, 0.046757]),
            np.abs(diffu_emb[:, 1]), 6)
        np.testing.assert_array_almost_equal(
            np.abs([0.023330, 0.040873, -0.064566, -0.075198, 0.088425]),
            np.abs(diffu_emb[:, 2]), 6)

    def test_get_random_walk_matrix(self):
        rw = np.array([
            [0.51517102, 0.18952083, 0.1832053, 0.06750703, 0.04459585],
            [0.22037146, 0.59903173, 0.0470527, 0.10012794, 0.03341618],
            [0.19758186, 0.04364105, 0.5555978, 0.10809274, 0.09508657],
            [0.07042714, 0.08983549, 0.1045631, 0.53745545, 0.19771881],
            [0.05148485, 0.03317748, 0.1017877, 0.21879730, 0.59475272],
        ])
        np.testing.assert_almost_equal(rw, get_randow_walk_matrix(self.gdm, k=2), 6)

    def test_get_gene_pseudoorder(self):
        np.testing.assert_array_equal([1, 2, 3, 4, 5], get_gene_pseudoorder(self.gdm, list(range(5)), 4))
        np.testing.assert_array_equal([5, 4, 3, 2, 1], get_gene_pseudoorder(self.gdm, list(range(5)), 0))
        np.testing.assert_array_equal([0, 3, 0, 2, 1], get_gene_pseudoorder(self.gdm, [1, 3, 4], 1))

    def test_extract_gene_trajectory(self):
        gt = extract_gene_trajectory(gene_embedding=self.gem, dist_mat=self.gdm, gene_names=gene_names,
                                     n=1, t_list=[3], dims=2)

        np.testing.assert_array_equal(gene_trajectories.index, gt.index)
        np.testing.assert_almost_equal(gene_trajectories['DM_1'], gt.DM_1.values, 6)
        np.testing.assert_almost_equal(gene_trajectories['DM_2'], gt.DM_2.values, 6)
        np.testing.assert_array_equal(gene_trajectories['selected'], gt.selected.values)
        np.testing.assert_array_equal(gene_trajectories['Pseudoorder-1'], gt['Pseudoorder-1'].values)


if __name__ == '__main__':
    unittest.main()



import unittest

import numpy as np

from gene_trajectory.gene_distance_shared import cal_ot_mat


class GeneDistanceSharedTestCase(unittest.TestCase):
    gdm = np.array([
        [0, 1, 2],
        [1, 0, 2],
        [2, 2, 1]])

    # Generated using R. Equal to get_gene_embedding() up to column signs
    gem = np.array([
        [.3, .1, .6],
        [.2, .3, .5],
        [.6, .2, .2]])

    expected_emd = np.array([
        [0.0, 0.8, 1.0],
        [0.8, 0.0, 0.9],
        [1.0, 0.9, 0.0]])

    gene_pairs = [(0, 1), (1, 1), (2, 1)]

    def test_gene_distance_shared(self):
        mt = cal_ot_mat(ot_cost=self.gdm, gene_expr=self.gem.T, show_progress_bar=False)
        np.testing.assert_almost_equal(self.expected_emd, mt, 6)

    def test_cal_ot_mat_gene_pairs(self):
        exp = self.expected_emd.copy()
        exp[0, 2] = exp[2, 0] = 900

        mt = cal_ot_mat(ot_cost=self.gdm, gene_expr=self.gem.T,
                        gene_pairs=self.gene_pairs,
                        show_progress_bar=False)
        np.testing.assert_almost_equal(exp, mt, 6)


if __name__ == '__main__':
    unittest.main()



import unittest

import numpy as np

from gene_trajectory.add_gene_bin_score import add_gene_bin_score
from tests.example_data import example_adata, diffusion_map, gene_trajectories, to_adata, gene_expression


class AddGeneScoreTestCase(unittest.TestCase):
    def test_add_gene_bin_score(self):

        adata = to_adata(gene_expression.T, var_names=list(gene_trajectories.index.values))

        add_gene_bin_score(adata, gene_trajectories, trajectories=1, n_bins=2)

        self.assertListEqual(['Trajectory1_genes1', 'Trajectory1_genes2'], adata.obs_keys())
        np.testing.assert_almost_equal([0.666667, 0.333333, 0.333333], adata.obs['Trajectory1_genes1'], decimal=5)
        np.testing.assert_almost_equal([0.5, 0.5, 0.5], adata.obs['Trajectory1_genes2'], decimal=5)


if __name__ == '__main__':
    unittest.main()



import numpy as np
import pandas as pd
import scanpy as sc
from scipy.sparse import csr_matrix

gene_expression = np.array([
    [2, 0, 1],
    [0, 3, 0],
    [3, 0, 0],
    [0, 0, 0],
    [1, 1, 1]
])

diffusion_map = np.array([
    [-0.04764353, 0.00940602, -0.00042641, -0.00158765],
    [0.11828658, 0.04134494, -0.00401907, -0.00012575],
    [-0.06615087, 0.03891922, -0.00681983,  0.00119593],
    [0.01417467, -0.05808308, -0.01944058,  0.0002601],
    [0.00654969, -0.02393814,  0.02780562,  0.00040004]
])

graph_distance = np.array([
    [0, 1, 1, 1, 1],
    [1, 0, 2, 1, 1],
    [1, 2, 0, 1, 1],
    [1, 1, 1, 0, 1],
    [1, 1, 1, 1, 0],
])

gene_names = ["Grin2a", "Sox2", "Cxcr4", "Cdkn1a", "Plk2"]

gene_trajectories = pd.DataFrame({
    'DM_1': [0.170435, 0.112734, 0.024611, -0.125216, -0.203563],
    'DM_2': [0.043091, -0.104203, 0.098184, -0.085159, 0.046757],
    'selected': ['Trajectory-1']*5,
    'Pseudoorder-1': [1, 2, 3, 4, 5],
}, index=gene_names)


def example_adata() -> sc.AnnData:
    return to_adata(gene_expression)


def to_adata(x: np.array, obs_names: list[str] = None, var_names: list[str] = None):
    adata = sc.AnnData(csr_matrix(np.asarray(x, dtype=np.float32)))
    adata.obs_names = obs_names or [f"Cell_{i:d}" for i in range(adata.n_obs)]
    adata.var_names = var_names or [f"Gene_{i:d}" for i in range(adata.n_vars)]
    return adata


def random_adata(shape=(100, 2000), seed=123) -> sc.AnnData:
    """
    A reasonably sized Scanpy element
    https://anndata.readthedocs.io/en/latest/tutorials/notebooks/getting-started.html
    """
    prng = np.random.RandomState(seed)
    counts = csr_matrix(prng.poisson(1, size=shape), dtype=np.float32)

    adata = sc.AnnData(counts)
    adata.obs_names = [f"Cell_{i:d}" for i in range(adata.n_obs)]
    adata.var_names = [f"Gene_{i:d}" for i in range(adata.n_vars)]

    adata.raw = adata
    if 'counts' not in adata.layers:
        adata.layers['counts'] = adata.raw.X.copy()

    return adata



import unittest

import numpy as np

from gene_trajectory.run_dm import run_dm
from tests.example_data import example_adata, diffusion_map


class RunDmTestCase(unittest.TestCase):
    @staticmethod
    def test_run_dm():
        adata = example_adata()
        run_dm(adata)

        np.testing.assert_almost_equal(adata.obsm['X_dm'], diffusion_map, decimal=5)


if __name__ == '__main__':
    unittest.main()



import unittest

import numpy as np
from sklearn.metrics import pairwise_distances

from gene_trajectory.coarse_grain import coarse_grain, select_top_genes
from tests.example_data import example_adata, random_adata


class CoarseGrainTestCase(unittest.TestCase):
    @staticmethod
    def test_cg():
        """
        Generated in R using
        https://github.com/KlugerLab/GeneTrajectory/blob/main/R/coarse.grain.R

        ```R
        random.seed = 1
        ce  = t(matrix(c(0,0, 2,3, 3,2,  5,0, 0,5), nrow = 2,
                dimnames = list(c("C1", "C2", "C3", "C4", "C5"), c("D1","D2"))))
        ge  = matrix(c(2,0,3,0,1, 0,3,0,0,1,  1,0,0,0,1), ncol = 3,
                     dimnames = list(c("C1", "C2", "C3", "C4", "C5"), c("G1","G2","G3")))
        cgd = as.matrix(dist(ce, method = "manhattan"))

        km.res <- stats::kmeans(ce, 4)
        km.res$cluster

        KNN.membership.mat <- matrix(0, nrow = 4, ncol = nrow(ce))
        for (i in 1:ncol(KNN.membership.mat)){
          KNN.membership.mat[km.res$cluster[i], i] <- 1
        }
        KNN.membership.mat <- KNN.membership.mat/apply(KNN.membership.mat, 1, sum)

        geu <- biclust::binarize(KNN.membership.mat, 0) %*% ge
        gdu <- KNN.membership.mat %*% cgd %*% t(KNN.membership.mat)
        ft  <- colnames(geu)
        ```
        """
        ce = np.array([[0, 0], [2, 3], [3, 2], [5, 0], [0, 5]])
        ge = np.array([[2, 0, 1], [0, 3, 0], [3, 0, 0], [0, 0, 0], [1, 1, 1]])
        n = 4

        gd = pairwise_distances(ce, metric='manhattan')

        cluster = np.array([2, 3, 3, 0, 1])
        gene_expression_updated, graph_dist_updated = coarse_grain(cell_embedding=ce,
                                                                   gene_expression=ge,
                                                                   graph_dist=gd,
                                                                   n=n,
                                                                   cluster=cluster)

        np.testing.assert_array_equal(np.array([[0, 0, 0], [1, 1, 1], [2, 0, 1], [3, 3, 0]]), gene_expression_updated)
        np.testing.assert_array_equal(np.array([[0, 10, 5, 5], [10, 0, 5, 5], [5, 5, 0, 5], [5, 5, 5, 1]]),
                                      graph_dist_updated)

    @staticmethod
    def test_cg2():
        """
        Same as test_cg but with a different k-mean result
        """
        ce = np.array([[0, 0], [2, 3], [3, 2], [5, 0], [0, 5]])
        ge = np.array([[2, 0, 1], [0, 3, 0], [3, 0, 0], [0, 0, 0], [1, 1, 1]])
        n = 4

        gd = pairwise_distances(ce, metric='manhattan')

        gene_expression_updated, graph_dist_updated = coarse_grain(cell_embedding=ce,
                                                                   gene_expression=ge,
                                                                   graph_dist=gd,
                                                                   n=n,
                                                                   random_seed=1)

        np.testing.assert_array_equal(np.array([[3, 3, 0], [1, 1, 1], [2, 0, 1], [0, 0, 0]]), gene_expression_updated)
        np.testing.assert_array_equal(np.array([[1, 5, 5, 5], [5, 0, 5, 10], [5, 5, 0, 5], [5, 10, 5, 0]]),
                                      graph_dist_updated)

    def test_select_top_genes(self):
        adata = random_adata(shape=(100, 3000), seed=123)

        genes = select_top_genes(adata, layer='counts')

        self.assertListEqual(['Gene_204', 'Gene_971', 'Gene_1915', 'Gene_1775'], genes.tolist())


if __name__ == '__main__':
    unittest.main()



import unittest

import numpy as np

from gene_trajectory.get_graph_distance import get_graph_distance
from tests.example_data import example_adata, diffusion_map, graph_distance


class CoarseGrainTestCase(unittest.TestCase):
    @staticmethod
    def test_get_graph_distance():
        adata = example_adata()
        adata.obsm['X_dm'] = diffusion_map

        gd = get_graph_distance(adata, k=3)
        np.testing.assert_almost_equal(gd, graph_distance, decimal=5)


if __name__ == '__main__':
    unittest.main()
